@@ -2,7 +2,9 @@ use onig::Regex;
 use pyo3::exceptions;
 use pyo3::prelude::*;
 
+/// Instantiate a new Regex with the given pattern
 #[pyclass(module = "tokenizers", name=Regex)]
+#[text_signature = "(self, pattern)"]
 pub struct PyRegex {
     pub inner: Regex,
     pub pattern: String,
