@@ -1,4 +1,5 @@
 #![warn(clippy::all)]
+#![allow(clippy::upper_case_acronyms)]
 #![doc(html_favicon_url = "https://huggingface.co/favicon.ico")]
 #![doc(html_logo_url = "https://huggingface.co/landing/assets/huggingface_logo.svg")]
 
@@ -43,7 +44,7 @@
 //! ```
 //!
 //! ## Training and serialization example
-//!  
+//!
 //! ```no_run
 //! use tokenizers::decoders::DecoderWrapper;
 //! use tokenizers::models::bpe::{BpeTrainerBuilder, BPE};
@@ -58,7 +59,7 @@
 //! fn main() -> Result<()> {
 //!     let vocab_size: usize = 100;
 //!
-//!     let trainer = BpeTrainerBuilder::new()
+//!     let mut trainer = BpeTrainerBuilder::new()
 //!         .show_progress(true)
 //!         .vocab_size(vocab_size)
 //!         .min_frequency(0)
@@ -71,7 +72,7 @@
 //!         ])
 //!         .build();
 //!
-//!     let tokenizer = TokenizerBuilder::new()
+//!     let mut tokenizer = TokenizerBuilder::new()
 //!         .with_model(BPE::default())
 //!         .with_normalizer(Some(Sequence::new(vec![
 //!             Strip::new(true, true).into(),
@@ -84,8 +85,8 @@
 //!
 //!     let pretty = false;
 //!     tokenizer
-//!         .train(
-//!             &trainer,
+//!         .train_from_files(
+//!             &mut trainer,
 //!             vec!["path/to/vocab.txt".to_string()],
 //!         )?
 //!         .save("tokenizer.json", pretty)?;
@@ -100,6 +101,11 @@
 //! by the total number of core/threads your CPU provides but this can be tuned by setting the `RAYON_RS_NUM_CPUS`
 //! environment variable. As an example setting `RAYON_RS_NUM_CPUS=4` will allocate a maximum of 4 threads.
 //! **_Please note this behavior may evolve in the future_**
+//!
+//! # Features
+//! **progressbar**: The progress bar visualization is enabled by default. It might be disabled if
+//!   compilation for certain targets is not supported by the [termios](https://crates.io/crates/termios)
+//!   dependency of the [indicatif](https://crates.io/crates/indicatif) progress bar.
 
 #[macro_use]
 extern crate log;
