@@ -1,6 +1,8 @@
 use crate::processors::byte_level::process_offsets;
 use crate::tokenizer::{Encoding, PostProcessor, Result};
 use serde::{Deserialize, Serialize};
+use std::collections::HashMap;
+use std::iter::FromIterator;
 
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
 #[serde(tag = "type")]
@@ -75,19 +77,22 @@ impl PostProcessor for RobertaProcessing {
             return PostProcessor::default_process(encoding, pair_encoding, add_special_tokens);
         }
 
-        let ids = [&[self.cls.1], &encoding.get_ids()[..], &[self.sep.1]].concat();
-        let type_ids = [&[0], &encoding.get_type_ids()[..], &[0]].concat();
+        let ids = [&[self.cls.1], encoding.get_ids(), &[self.sep.1]].concat();
+        let type_ids = [&[0], encoding.get_type_ids(), &[0]].concat();
         let tokens = [
             &[self.cls.0.clone()],
-            &encoding.get_tokens()[..],
+            encoding.get_tokens(),
             &[self.sep.0.clone()],
         ]
         .concat();
-        let words = [&[None], &encoding.get_words()[..], &[None]].concat();
-        let offsets = [&[(0, 0)], &encoding.get_offsets()[..], &[(0, 0)]].concat();
+        let words = [&[None], encoding.get_word_ids(), &[None]].concat();
+        let offsets = [&[(0, 0)], encoding.get_offsets(), &[(0, 0)]].concat();
         let special_tokens = [&[1u32], &vec![0; encoding.get_ids().len()][..], &[1]].concat();
         let attention_mask = vec![1; ids.len()];
 
+        // For compatibility with `TemplateProcessing`, the sequence_ranges shouldn't contain
+        // the special tokens.
+        let sequence_ranges = HashMap::from_iter(vec![(0, 1..ids.len() - 1)]);
         let mut new_encoding = Encoding::new(
             ids,
             type_ids,
@@ -100,20 +105,23 @@ impl PostProcessor for RobertaProcessing {
                 .take_overflowing()
                 .into_iter()
                 .map(|encoding| {
-                    let ids = [&[self.cls.1], &encoding.get_ids()[..], &[self.sep.1]].concat();
-                    let type_ids = [&[0], &encoding.get_type_ids()[..], &[0]].concat();
+                    let ids = [&[self.cls.1], encoding.get_ids(), &[self.sep.1]].concat();
+                    let type_ids = [&[0], encoding.get_type_ids(), &[0]].concat();
                     let tokens = [
                         &[self.cls.0.clone()],
-                        &encoding.get_tokens()[..],
+                        encoding.get_tokens(),
                         &[self.sep.0.clone()],
                     ]
                     .concat();
-                    let words = [&[None], &encoding.get_words()[..], &[None]].concat();
-                    let offsets = [&[(0, 0)], &encoding.get_offsets()[..], &[(0, 0)]].concat();
+                    let words = [&[None], encoding.get_word_ids(), &[None]].concat();
+                    let offsets = [&[(0, 0)], encoding.get_offsets(), &[(0, 0)]].concat();
                     let special_tokens =
                         [&[1u32], &vec![0; encoding.get_ids().len()][..], &[1]].concat();
                     let attention_mask = vec![1; ids.len()];
 
+                    // For compatibility with `TemplateProcessing`, the sequence_ranges shouldn't
+                    // contain the special tokens.
+                    let sequence_ranges = HashMap::from_iter(vec![(0, 1..ids.len() - 1)]);
                     Encoding::new(
                         ids,
                         type_ids,
@@ -123,26 +131,31 @@ impl PostProcessor for RobertaProcessing {
                         special_tokens,
                         attention_mask,
                         vec![],
+                        sequence_ranges,
                     )
                 })
                 .collect(),
+            sequence_ranges,
         );
 
         if let Some(mut encoding) = pair_encoding {
-            let pair_ids = [&[self.sep.1], &encoding.get_ids()[..], &[self.sep.1]].concat();
+            let pair_ids = [&[self.sep.1], encoding.get_ids(), &[self.sep.1]].concat();
             let pair_type_ids = vec![0; encoding.get_ids().len() + 2];
             let pair_tokens = [
                 &[self.sep.0.clone()],
-                &encoding.get_tokens()[..],
+                encoding.get_tokens(),
                 &[self.sep.0.clone()],
             ]
             .concat();
-            let pair_words = [&[None], &encoding.get_words()[..], &[None]].concat();
-            let pair_offsets = [&[(0, 0)], &encoding.get_offsets()[..], &[(0, 0)]].concat();
+            let pair_words = [&[None], encoding.get_word_ids(), &[None]].concat();
+            let pair_offsets = [&[(0, 0)], encoding.get_offsets(), &[(0, 0)]].concat();
             let pair_special_tokens =
                 [&[1], &vec![0u32; encoding.get_type_ids().len()][..], &[1]].concat();
             let pair_attention_mask = vec![1; pair_ids.len()];
 
+            // For compatibility with `TemplateProcessing`, the sequence_ranges shouldn't contain
+            // the special tokens.
+            let pair_sequence_ranges = HashMap::from_iter(vec![(1, 1..pair_ids.len() - 1)]);
             let new_pair_encoding = Encoding::new(
                 pair_ids,
                 pair_type_ids,
@@ -155,22 +168,24 @@ impl PostProcessor for RobertaProcessing {
                     .take_overflowing()
                     .into_iter()
                     .map(|encoding| {
-                        let pair_ids =
-                            [&[self.sep.1], &encoding.get_ids()[..], &[self.sep.1]].concat();
+                        let pair_ids = [&[self.sep.1], encoding.get_ids(), &[self.sep.1]].concat();
                         let pair_type_ids = vec![0; encoding.get_ids().len() + 2];
                         let pair_tokens = [
                             &[self.sep.0.clone()],
-                            &encoding.get_tokens()[..],
+                            encoding.get_tokens(),
                             &[self.sep.0.clone()],
                         ]
                         .concat();
-                        let pair_words = [&[None], &encoding.get_words()[..], &[None]].concat();
-                        let pair_offsets =
-                            [&[(0, 0)], &encoding.get_offsets()[..], &[(0, 0)]].concat();
+                        let pair_words = [&[None], encoding.get_word_ids(), &[None]].concat();
+                        let pair_offsets = [&[(0, 0)], encoding.get_offsets(), &[(0, 0)]].concat();
                         let pair_special_tokens =
                             [&[1], &vec![0u32; encoding.get_type_ids().len()][..], &[1]].concat();
                         let pair_attention_mask = vec![1; pair_ids.len()];
 
+                        // For compatibility with `TemplateProcessing`, the sequence_ranges
+                        // shouldn't contain the special tokens.
+                        let pair_sequence_ranges =
+                            HashMap::from_iter(vec![(1, 1..pair_ids.len() - 1)]);
                         Encoding::new(
                             pair_ids,
                             pair_type_ids,
@@ -180,9 +195,11 @@ impl PostProcessor for RobertaProcessing {
                             pair_special_tokens,
                             pair_attention_mask,
                             vec![],
+                            pair_sequence_ranges,
                         )
                     })
                     .collect(),
+                pair_sequence_ranges,
             );
 
             new_encoding.merge_with(new_pair_encoding, false);
