@@ -19,12 +19,12 @@ use std::{
     path::{Path, PathBuf},
 };
 
-use indicatif::{ProgressBar, ProgressStyle};
 use serde::de::DeserializeOwned;
-use serde::export::Formatter;
 use serde::{Deserialize, Serialize};
 
+use crate::utils::iter::ResultShunt;
 use crate::utils::parallelism::*;
+use crate::utils::progress::{ProgressBar, ProgressStyle};
 
 mod added_vocabulary;
 mod encoding;
@@ -68,20 +68,23 @@ pub trait PreTokenizer {
 
 /// Represents a model used during Tokenization (like BPE or Word or Unigram).
 pub trait Model {
+    type Trainer: Trainer + Sync;
     /// Tokenize the given sequence into multiple underlying `Token`. The `offsets` on the `Token`
     /// are expected to be relative to the given sequence.
     fn tokenize(&self, sequence: &str) -> Result<Vec<Token>>;
     /// Find the ID associated to a string token
     fn token_to_id(&self, token: &str) -> Option<u32>;
     /// Find the string token associated to an ID
-    fn id_to_token(&self, id: u32) -> Option<&str>;
+    fn id_to_token(&self, id: u32) -> Option<String>;
     /// Retrieve the entire vocabulary mapping (token -> ID)
-    fn get_vocab(&self) -> &HashMap<String, u32>;
+    fn get_vocab(&self) -> HashMap<String, u32>;
     /// Retrieve the size of the vocabulary
     fn get_vocab_size(&self) -> usize;
     /// Save the current `Model` in the given folder, using the given `prefix` for the various
     /// files that need to be saved.
     fn save(&self, folder: &Path, prefix: Option<&str>) -> Result<Vec<PathBuf>>;
+    /// Get an instance of a Trainer capable of training this Model
+    fn get_trainer(&self) -> <Self as Model>::Trainer;
 }
 
 /// A `PostProcessor` has the responsibility to post process an encoded output of the `Tokenizer`.
@@ -121,19 +124,21 @@ pub trait Decoder {
 }
 
 /// A `Trainer` has the responsibility to train a model. We feed it with lines/sentences
-/// and it returns a `Model` when done.
+/// and then it can train the given `Model`.
 pub trait Trainer {
     type Model: Model + Sized;
     /// Whether we should show progress during the training.
     fn should_show_progress(&self) -> bool;
     /// The actual training method. This will return a new trained Model as well as a list
     /// of `special_tokens` to be added directly to the tokenizer along with the model.
-    fn train(
-        &self,
-        words: HashMap<String, u32>,
-    ) -> Result<(<Self as Trainer>::Model, Vec<AddedToken>)>;
-    /// Process a bunch of token, counting them as relevant.
-    fn process_tokens(&self, words: &mut HashMap<String, u32>, tokens: Vec<String>);
+    fn train(&self, model: &mut Self::Model) -> Result<Vec<AddedToken>>;
+    /// Process an iterator of sequences, calling `process` for each of them in order to
+    /// pre-process the said sequence as relevant.
+    fn feed<I, S, F>(&mut self, iterator: I, process: F) -> Result<()>
+    where
+        I: Iterator<Item = S> + Send,
+        S: AsRef<str> + Send,
+        F: Fn(&str) -> Result<Vec<String>> + Sync;
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -239,7 +244,7 @@ pub struct BuilderError(String);
 impl std::error::Error for BuilderError {}
 
 impl fmt::Display for BuilderError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.0)
     }
 }
@@ -574,7 +579,7 @@ where
 
     /// Get the vocabulary
     pub fn get_vocab(&self, with_added_tokens: bool) -> HashMap<String, u32> {
-        let mut final_vocab = self.model.get_vocab().clone();
+        let mut final_vocab = self.model.get_vocab();
 
         if with_added_tokens {
             let added_vocab = self.added_vocabulary.get_vocab();
@@ -605,7 +610,7 @@ where
     }
 
     /// Converts an id to the corresponding token.
-    pub fn id_to_token(&self, id: u32) -> Option<&str> {
+    pub fn id_to_token(&self, id: u32) -> Option<String> {
         self.added_vocabulary.id_to_token(id, &self.model)
     }
 
@@ -689,10 +694,9 @@ where
 
         // Encode each sequence
         let encoding = self.encode_single_sequence(sequence, 0, OffsetType::Byte)?;
-        let pair_encoding = match pair {
-            Some(sequence) => Some(self.encode_single_sequence(sequence, 1, OffsetType::Byte)?),
-            None => None,
-        };
+        let pair_encoding = pair
+            .map(|sequence| self.encode_single_sequence(sequence, 1, OffsetType::Byte))
+            .transpose()?;
 
         // And finally post process
         self.post_process(encoding, pair_encoding, add_special_tokens)
@@ -733,10 +737,9 @@ where
 
         // Encode each sequence
         let encoding = self.encode_single_sequence(sequence, 0, OffsetType::Char)?;
-        let pair_encoding = match pair {
-            Some(sequence) => Some(self.encode_single_sequence(sequence, 1, OffsetType::Char)?),
-            None => None,
-        };
+        let pair_encoding = pair
+            .map(|sequence| self.encode_single_sequence(sequence, 1, OffsetType::Char))
+            .transpose()?;
 
         // And finally post process
         self.post_process(encoding, pair_encoding, add_special_tokens)
@@ -752,7 +755,6 @@ where
                     .filter(|token| {
                         !skip_special_tokens || !self.added_vocabulary.is_special_token(token)
                     })
-                    .map(|t| t.to_owned())
             })
             .collect::<Vec<_>>();
 
@@ -959,13 +961,11 @@ where
             .collect()
     }
 
-    /// Train a model and replace our current Model, using the given Trainer
-    fn word_count<MN, T>(&self, trainer: &T, files: Vec<String>) -> Result<HashMap<String, u32>>
+    /// Train our Model from files
+    pub fn train_from_files<T>(&mut self, trainer: &mut T, files: Vec<String>) -> Result<&mut Self>
     where
-        T: Trainer<Model = MN> + Sync,
-        MN: Model,
+        T: Trainer<Model = M> + Sync,
     {
-        let max_read = 1_000_000;
         let mut len = 0;
         for file in files.iter() {
             len += File::open(file)
@@ -973,118 +973,118 @@ where
                 .map(|m| m.len())?;
         }
 
+        let max_read = 1_000_000;
+
+        ResultShunt::process(
+            files.into_iter().flat_map(|filename| {
+                match File::open(filename) {
+                    Ok(file) => {
+                        let file = BufReader::with_capacity(max_read, file);
+                        // We read new lines using this API instead of the Lines Iterator
+                        // on purpose. We want to keep the `\n` and potential `\r` between each lines
+                        // We use an iterator to be able to chain with par_bridge.
+                        itertools::Either::Left(file.lines_with_ending())
+                    }
+                    Err(e) => itertools::Either::Right(std::iter::once(Err(e))),
+                }
+            }),
+            |sequences| -> Result<()> {
+                let progress = if trainer.should_show_progress() {
+                    let progress = ProgressBar::new(len);
+                    progress.set_style(
+                        ProgressStyle::default_bar()
+                            .template("[{elapsed_precise}] {msg:<40!} {wide_bar} {percent:>18!}%"),
+                    );
+                    progress
+                        .set_message(&format!("Pre-processing files ({:.2} Mo)", len / 1_000_000));
+                    progress.set_draw_delta(len / 100); // Redraw only every 2%
+                    Some(progress)
+                } else {
+                    None
+                };
+
+                trainer.feed(
+                    sequences.map(|s| {
+                        if let Some(progress) = &progress {
+                            progress.inc(s.len() as u64)
+                        }
+                        s
+                    }),
+                    |seq| {
+                        let normalized = self.do_normalize(seq.as_ref())?;
+                        let pre_tokenized = self.do_pre_tokenize(normalized)?;
+                        Ok(pre_tokenized
+                            .get_splits(OffsetReferential::Original, OffsetType::Byte)
+                            .into_iter()
+                            .map(|(s, _, _)| s.to_owned())
+                            .collect())
+                    },
+                )?;
+
+                if let Some(pbar) = progress {
+                    pbar.finish();
+                }
+                let special_tokens = trainer.train(&mut self.model)?;
+                self.add_special_tokens(&special_tokens);
+
+                Ok(())
+            },
+        )??;
+        Ok(self)
+    }
+
+    /// Train our Model, using the given Trainer and iterator
+    pub fn train<T, I, S>(&mut self, trainer: &mut T, sequences: I) -> Result<&mut Self>
+    where
+        T: Trainer<Model = M> + Sync,
+        I: Iterator<Item = S> + Send,
+        S: AsRef<str> + Send,
+    {
+        let (lower, upper) = sequences.size_hint();
+        let len = upper.unwrap_or(lower) as u64;
         let progress = if trainer.should_show_progress() {
             let progress = ProgressBar::new(len);
             progress.set_style(
                 ProgressStyle::default_bar()
-                    .template("[{elapsed_precise}] {msg:<40!} {wide_bar} {percent:>19!}"),
+                    .template("[{elapsed_precise}] {msg:<40!} {wide_bar} {pos:<9!}/{len:>9!}"),
             );
-            progress.set_message(&format!("Reading files ({:.2} Mo)", len / 1_000_000));
-            progress.set_draw_delta(len / 100); // Redraw only every 2%
+            progress.set_message("Pre-processing sequences");
+            if len > 0 {
+                progress.set_draw_delta(len / 100); // Redraw only every 2%
+            } else {
+                // Trying to have a good default to avoid progress tracking being the bottleneck
+                progress.set_draw_delta(1000);
+            }
             Some(progress)
         } else {
             None
         };
-        let words = files
-            .into_iter()
-            .map(|filename| -> Result<HashMap<String, u32>> {
-                let file = File::open(filename)?;
-                let file = BufReader::with_capacity(max_read, file);
-                // We read new lines using this API instead of the Lines Iterator
-                // on purpose. We want to keep the `\n` and potential `\r` between each lines
-                // We use an iterator to be able to chain with par_bridge.
-                file.lines_with_ending()
-                    .maybe_par_bridge()
-                    .map_with(
-                        &progress,
-                        |progress, line| -> Result<HashMap<String, u32>> {
-                            let newline = line?;
-                            let b = newline.len();
-                            let mut words = HashMap::new();
-                            let normalized = self.do_normalize(newline)?;
-                            let pre_tokenized = self.do_pre_tokenize(normalized)?;
-                            trainer.process_tokens(
-                                &mut words,
-                                pre_tokenized
-                                    .get_splits(OffsetReferential::Original, OffsetType::Byte)
-                                    .into_iter()
-                                    .map(|(s, _, _)| s.to_owned())
-                                    .collect(),
-                            );
 
-                            if let Some(pbar) = progress {
-                                pbar.inc(b as u64);
-                            }
-                            Ok(words)
-                        },
-                    )
-                    .reduce(
-                        || Ok(HashMap::new()),
-                        |acc, ws| {
-                            let mut acc = acc?;
-                            for (k, v) in ws? {
-                                acc.entry(k).and_modify(|c| *c += v).or_insert(v);
-                            }
-                            Ok(acc)
-                        },
-                    )
-            })
-            .try_fold(
-                HashMap::new(),
-                |mut acc, ws| -> Result<HashMap<String, u32>> {
-                    for (k, v) in ws? {
-                        acc.entry(k).and_modify(|c| *c += v).or_insert(v);
-                    }
-                    Ok(acc)
-                },
-            )?;
+        trainer.feed(
+            sequences.map(|s| {
+                if let Some(progress) = &progress {
+                    progress.inc(1)
+                }
+                s
+            }),
+            |seq| {
+                let normalized = self.do_normalize(seq.as_ref())?;
+                let pre_tokenized = self.do_pre_tokenize(normalized)?;
+                Ok(pre_tokenized
+                    .get_splits(OffsetReferential::Original, OffsetType::Byte)
+                    .into_iter()
+                    .map(|(s, _, _)| s.to_owned())
+                    .collect())
+            },
+        )?;
         if let Some(pbar) = progress {
             pbar.finish();
         }
-        Ok(words)
-    }
 
-    /// Train a model and return a new Tokenizer, using the given Trainer
-    pub fn train<T, TM>(
-        self,
-        trainer: &T,
-        files: Vec<String>,
-    ) -> Result<TokenizerImpl<TM, N, PT, PP, D>>
-    where
-        T: Trainer<Model = TM> + Sync,
-        TM: Model,
-    {
-        let words = self.word_count(trainer, files)?;
-
-        let (model, special_tokens) = trainer.train(words)?;
-        let mut new_tok = TokenizerImpl {
-            normalizer: self.normalizer,
-            pre_tokenizer: self.pre_tokenizer,
-            model,
-            post_processor: self.post_processor,
-            decoder: self.decoder,
-            added_vocabulary: self.added_vocabulary,
-            truncation: self.truncation,
-            padding: self.padding,
-        };
-
-        new_tok.add_special_tokens(&special_tokens);
-
-        Ok(new_tok)
-    }
-
-    /// Train a model and replace our current Model, using the given Trainer
-    pub fn train_and_replace<T>(&mut self, trainer: &T, files: Vec<String>) -> Result<()>
-    where
-        T: Trainer<Model = M> + Sync,
-    {
-        let words = self.word_count(trainer, files)?;
-
-        let (model, special_tokens) = trainer.train(words)?;
-        self.model = model;
+        let special_tokens = trainer.train(&mut self.model)?;
         self.add_special_tokens(&special_tokens);
 
-        Ok(())
+        Ok(self)
     }
 }
 
