@@ -8,20 +8,25 @@ use rayon_cond::CondIterator;
 
 pub const ENV_VARIABLE: &str = "TOKENIZERS_PARALLELISM";
 
+// Reading/Writing this variable should always happen on the main thread
+static mut USED_PARALLELISM: bool = false;
+
 /// Check if the TOKENIZERS_PARALLELISM env variable has been explicitly set
 pub fn is_parallelism_configured() -> bool {
     std::env::var(ENV_VARIABLE).is_ok()
 }
 
+/// Check if at some point we used a parallel iterator
+pub fn has_parallelism_been_used() -> bool {
+    unsafe { USED_PARALLELISM }
+}
+
 /// Get the currently set value for `TOKENIZERS_PARALLELISM` env variable
 pub fn get_parallelism() -> bool {
     match std::env::var(ENV_VARIABLE) {
         Ok(mut v) => {
             v.make_ascii_lowercase();
-            match v.as_ref() {
-                "" | "off" | "false" | "f" | "no" | "n" | "0" => false,
-                _ => true,
-            }
+            !matches!(v.as_ref(), "" | "off" | "false" | "f" | "no" | "n" | "0")
         }
         Err(_) => true, // If we couldn't get the variable, we use the default
     }
@@ -60,7 +65,11 @@ where
     S: Iterator<Item = P::Item>,
 {
     fn into_maybe_par_iter(self) -> CondIterator<P, S> {
-        CondIterator::new(self, get_parallelism())
+        let parallelism = get_parallelism();
+        if parallelism {
+            unsafe { USED_PARALLELISM = true };
+        }
+        CondIterator::new(self, parallelism)
     }
 
     fn into_maybe_par_iter_cond(self, cond: bool) -> CondIterator<P, S> {
@@ -147,6 +156,7 @@ where
         let iter = CondIterator::from_serial(self);
 
         if get_parallelism() {
+            unsafe { USED_PARALLELISM = true };
             CondIterator::from_parallel(iter.into_parallel().right().unwrap())
         } else {
             iter
@@ -167,12 +177,19 @@ mod tests {
     use super::*;
 
     #[test]
-    #[ignore]
     fn test_maybe_parallel_iterator() {
         let mut v = vec![1u32, 2, 3, 4, 5, 6];
 
         assert_eq!(v.maybe_par_iter().sum::<u32>(), 21);
-        assert_eq!(v.maybe_par_iter_mut().map(|v| *v * 2).sum::<u32>(), 42);
+        assert_eq!(
+            v.maybe_par_iter_mut()
+                .map(|v| {
+                    *v *= 2;
+                    *v
+                })
+                .sum::<u32>(),
+            42
+        );
         assert_eq!(v.maybe_par_iter().sum::<u32>(), 42);
         assert_eq!(v.into_maybe_par_iter().sum::<u32>(), 42);
     }
